# -*- coding: utf-8 -*-

# Copyright 2015 FUJITSU LIMITED
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_ism_base
----------------------------------

Tests for `networking_fujitsu` module.
"""

from networking_fujitsu.tests import base
# from networking_fujitsu.ml2.drivers.fujitsu.ism import ism_base


class FujitsuIsmVlanBaseTestCase(base.TestCase):

    def setUp(self):
        super(FujitsuIsmVlanBaseTestCase, self).setUp()
        pass

    def tearDown(self):
        super(FujitsuIsmVlanBaseTestCase, self).tearDown()
        pass

    def test_something(self):
        pass
